;pulse train generator to test "sdsound-tn13" direct title selection
;ATtiny2313 @ 1 MHz (internal R/C oscillator), lfuse=0x44, hfuse=0xDB
;
;PORTS:
;  PD2 = output (to #RESET pin of ATtiny13, open drain)
;  PD3 = key "4"
;  PD4 = key "7"
;  PD5 = key "*"
;  PD6 = key "1"
;  PB0 = key "0"
;  PB1 = key "8"
;  PB2 = key "5"
;  PB3 = key "2"
;  PB4 = key "#"
;  PB5 = key "9"
;  PB6 = key "6"
;  PB7 = key "3"
;  (all keys to ground)
;
;Press single number key to select titles 0~9.
;Press *, then two number keys to select titles 10~63.
;
;Arne Rossius, 2013-04-28

.equ	kAsterisk = 0x10
.equ	kHash = 0x11
.equ	kNone = 0xFF

;===============================================================================

.include "tn2313def.inc"

.def	temp = R16
.def	temp2 = R17
.def	count = R18
.def	key_count = R19
.def	last_key = R20
.def	keypress = R21

;===============================================================================

.cseg
.org 0x000
	rjmp	reset
.org OVF0addr
	rjmp	ovf0
	
;===============================================================================

ovf0:
	;timer 0 overflow interrupt (488 Hz)
	push	temp
	in	temp, SREG
	push	temp
	
	;poll keypad
	ldi	temp, kNone
	sbis	PINB, 0
	ldi	temp, 0
	sbis	PINB, 1
	ldi	temp, 8
	sbis	PINB, 2
	ldi	temp, 5
	sbis	PINB, 3
	ldi	temp, 2
	sbis	PINB, 4
	ldi	temp, kHash
	sbis	PINB, 5
	ldi	temp, 9
	sbis	PINB, 6
	ldi	temp, 6
	sbis	PINB, 7
	ldi	temp, 3
	sbis	PIND, 3
	ldi	temp, 4
	sbis	PIND, 4
	ldi	temp, 7
	sbis	PIND, 5
	ldi	temp, kAsterisk
	sbis	PIND, 6
	ldi	temp, 1
	cpi	temp, kNone
	breq	ovf0_released
	cp	last_key, temp
	breq	ovf0_no_change
	
	;new key pressed
	clr	key_count
	rjmp	ovf0_end

ovf0_released:
	;no key pressed
	clr	key_count
	rjmp	ovf0_end

ovf0_no_change:
	;no change since last interrupt
	cpi	key_count, 255 ;prevent overflow
	breq	PC+2
	inc	key_count
	cpi	key_count, 3 ;pressed for 15 ms
	brne	PC+2
	mov	keypress, temp
	
ovf0_end:
	mov	last_key, temp
	
	;return
	pop	temp
	out	SREG, temp
	pop	temp
	reti
	
;===============================================================================

reset:
	;init stackpointer
	ldi	temp, RAMEND
	out	SPL, temp
	
	;init ports
	ldi	temp, 0x00
	out	DDRB, temp
	ldi	temp, 0xFF
	out	PORTB, temp
	ldi	temp, 0x02
	out	DDRD, temp
	ldi	temp, 0x7B
	out	PORTD, temp
	ldi	temp, 0x00
	out	DDRA, temp
	ldi	temp, 0x03
	out	PORTA, temp
	
	;init timer 0
	ldi	temp, 0x02 ;Clk/8 => ~488 Hz overflow
	out	TCCR0B, temp
	ldi	temp, 1<<TOIE0 ;enable overflow interrupt
	out	TIMSK, temp
	
	;init registers
	ldi	keypress, kNone
	ldi	last_key, kNone
	ldi	key_count, 0
	
	;enable interrupts
	sei
	
;-------------------------------------------------------------------------------

main_loop:
	cpi	keypress, 10
	brlo	key_number
	cpi	keypress, kAsterisk ;two-digit key
	breq	two_digits
	rjmp	main_loop
	
key_number:
	;single number: select titles 0~9
	mov	count, keypress
	rcall	select_title
	ldi	keypress, kNone
	rjmp	main_loop
	
two_digits:
	;two-digit key pressed: wait for two number keys to select titles 10~63
	ldi	keypress, kNone
first_digit:
	cpi	keypress, 10
	brsh	first_digit
	mov	temp, keypress
	ldi	keypress, kNone
	lsl	temp
	mov	count, temp
	lsl	temp
	lsl	temp
	add	count, temp
second_digit:
	cpi	keypress, 10
	brsh	second_digit
	add	count, keypress
	cpi	count, 64
	brlo	PC+2
	ldi	count, 64 ;title "64" = stop
	rcall	select_title
	ldi	keypress, kNone
	rjmp	main_loop
	
;===============================================================================
	
select_title:
	;send pulse train to select title
	subi	count, -2
pulse_loop:
	sbi	DDRD, 2
	rcall	delay_pulse
	cbi	DDRD, 2
	rcall	delay_5ms
	dec	count
	brne	pulse_loop
	
;--------------------

delay_5ms:
	ldi	temp, 5
	ldi	temp2, 0
delay_5ms_loop:
	nop
	dec	temp2
	brne	delay_5ms_loop
	dec	temp
	brne	delay_5ms_loop
	ret
	
;--------------------

delay_pulse:
	ldi	temp, 5
delay_pulse_loop:
	dec	temp
	brne	delay_pulse_loop
	ret
	